(* Constructions with Setoids: Sums, Quotients, etc...  *)
(* by Venanzio Capretta                                 *)
(* Coq version 6.2.3                                    *)

Require Export Setoids.

(* Quotient of a Setoid by an equivalence relation. *)

(* We show how the usual rules for quotients can be obtained.          *)
(* See martin Hofmann, A Simple Model for Quotient Types.              *)
(* Contrary to Hofmann we take quotient only by equivalence relations. *)

(* Q-Form *)
Definition Quotient : (S:Setoid)(Setoid_equivalence S) -> Setoid :=
  [S,R](setoid (se_proof R)).

Infix 8 "/" Quotient.

Implicit Arguments On.

Section Setoid_Quotients.

Variable S   : Setoid.
Variable R   : (Setoid_equivalence S).

Local Q := S/R.

(* Q-Intro *)
Definition class : S -> Q := [x:S]x.

(* Q-Elim *)
Definition plug :
  (T:Setoid)(f : S => T)((x,y:S)(R x y) -> (f x) ]=[ (f y)) ->
  (Q => T)
:=
  [T,f,H]
   (setoid_function [x1,x2:Q][p:(x1]=[x2)](H x1 x2 p)).

(* Q-Ax *)
Lemma Qax : (x,y:S)(R x y) -> (class x) ]=[ (class y).

Proof. Trivial. Qed.

(* Q-Ind *)
Lemma Qind :
  (P : (Setoid_predicate Q))
  ((x:S)(P (class x))) ->
  (x:Q)(P x).

Proof. Trivial. Qed.

End Setoid_Quotients.

Implicit Arguments Off.

